package com.codigoparallevar.minicards.bridge.blocks;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.graphics.Color;
import android.os.Build;

import androidx.core.app.NotificationCompat;

import com.codigoparallevar.minicards.R;
import com.codigoparallevar.minicards.bridge.ProgramakerBridgeService;
import com.codigoparallevar.minicards.types.functional.Box;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

public class DefaultAndroidBlocks {

    public static BridgeBlockListBuilder GetBuilder(Context ctx) {
        // System services
        String notifServiceId = Context.NOTIFICATION_SERVICE;
        NotificationManager notificationManager = (NotificationManager) ctx.getSystemService(notifServiceId);
        assert notificationManager != null;

        Random notificationRandom = new Random();
        int ledsNotificationId = notificationRandom.nextInt();
        Box<Integer> currentLedArg = new Box<>(0x00000000);

        NotificationChannel channelPreparation = null;
        NotificationChannel ledsChannelPreparation = null;

        String notificationChannelId = ProgramakerBridgeService.BridgeUserNotificationChannel;
        String ledsChannelId = ProgramakerBridgeService.BridgeUserLedsNotificationChannel;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            channelPreparation = new NotificationChannel(
                    ProgramakerBridgeService.BridgeUserNotificationChannel,
                    ProgramakerBridgeService.BridgeUserNotificationChannelName,
                    NotificationManager.IMPORTANCE_DEFAULT);
            notificationManager.createNotificationChannel(channelPreparation);

            ledsChannelPreparation = new NotificationChannel(
                    ledsChannelId,
                    ProgramakerBridgeService.BridgeUserLedsNotificationChannelName,
                    NotificationManager.IMPORTANCE_DEFAULT);

            ledsChannelPreparation.enableLights(true);

            notificationManager.createNotificationChannel(ledsChannelPreparation);
        }

        final NotificationChannel notificationChannel = channelPreparation;
        final NotificationChannel ledsChannel = ledsChannelPreparation;
        return new BridgeBlockListBuilder()
                // Notifications
                .addOperation(
                        "notifications_new",
                        "Create notification. Title: %1, text: %2",
                        new LinkedList<BlockArgumentDefinition>() {{
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.STRING, "My notification"));
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.STRING, "Sample description"));
                        }},
                        (List<?> params) -> {
                            if (params.size() != 2) {
                                throw new Exception("Expected two (2) arguments, found " + params.size());
                            }
                            String title = params.get(0).toString();
                            String description = params.get(1).toString();

                            Notification notif = new NotificationCompat
                                    .Builder(ctx, ProgramakerBridgeService.BridgeUserNotificationChannel)
                                    .setContentTitle(title)
                                    .setContentText(description)
                                    .setSmallIcon(R.drawable.ic_center_focus_weak_black) // TODO: Change icon
                                    .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                                    .build();

                            int notificationId = notificationRandom.nextInt();
                            notificationManager.notify(notificationId, notif);
                        }
                )
                .addOperation(
                        "notifications_clear",
                        "Clear notifications",
                        Collections.emptyList(),
                        (List<?> params) -> {
                            notificationManager.cancelAll();
                        }
                )
                // Leds
                .addOperation(
                        "leds_turn_on",
                        "Set leds to (r:%1,g:%2,b:%3)",
                        new LinkedList<BlockArgumentDefinition>() {{
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.INT, "255"));
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.INT, "255"));
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.INT, "255"));
                        }},
                        params -> {
                            if (params.size() != 3) {
                                throw new Exception("Expected three (3) arguments, found " + params.size());
                            }
                            String rStr = params.get(0).toString();
                            String gStr = params.get(1).toString();
                            String bStr = params.get(2).toString();

                            int r = Math.min(255, Math.max(0, Integer.parseInt(rStr)));
                            int g = Math.min(255, Math.max(0, Integer.parseInt(gStr)));
                            int b = Math.min(255, Math.max(0, Integer.parseInt(bStr)));

                            int ledColor = 0x7f * (1<<24) + r * (1 << 16) + g * (1 << 8) + b;

                            Notification notif = new NotificationCompat
                                    .Builder(ctx, ledsChannelId)
                                    .setPriority(NotificationCompat.PRIORITY_LOW)
                                    .setContentTitle(ctx.getString(R.string.leds_notification))
                                    .setSmallIcon(R.drawable.ic_center_focus_weak_black) // TODO: Change icon
                                    .setLights(Color.RED, 0, 0)
                                    .build();

                            notif.ledARGB = Color.RED;
                            notif.ledOnMS = 300;
                            notif.ledOffMS = 300;

                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                                ledsChannel.setLightColor(Color.RED);
                            }

                            notificationManager.notify(ledsNotificationId, notif);
                        }
                )
                ;
    }
}
